-----------------------------------------------------------------------------------------------
-- Client Lua Script for ExportRoster
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"

Utils = { }
Utils.ClassNames = {
  [GameLib.CodeEnumClass.Warrior]      = "Warrior",
  [GameLib.CodeEnumClass.Engineer]     = "Engineer",
  [GameLib.CodeEnumClass.Esper]        = "Esper",
  [GameLib.CodeEnumClass.Medic]        = "Medic",
  [GameLib.CodeEnumClass.Stalker]      = "Stalker",
  [GameLib.CodeEnumClass.Spellslinger] = "Spellslinger"
}
 
-----------------------------------------------------------------------------------------------
-- ExportRoster Module Definition
-----------------------------------------------------------------------------------------------
local ExportRoster = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ExportRoster:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ExportRoster:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)

	self.scanning = false;
	self.timer = ApolloTimer.Create(3, true, "OnTimer", self)
end
 

-----------------------------------------------------------------------------------------------
-- ExportRoster OnLoad
-----------------------------------------------------------------------------------------------
function ExportRoster:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ExportRoster.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ExportRoster OnDocLoaded
-----------------------------------------------------------------------------------------------
function ExportRoster:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ExportRosterForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("exportroster", "OnExportRoster", self)
		Apollo.RegisterSlashCommand("ExportRoster", "OnExportRoster", self)
		Apollo.RegisterSlashCommand("export", "OnExport", self)
		Apollo.RegisterSlashCommand("Export", "OnExport", self)
		Apollo.RegisterEventHandler("Inspect","OnInspect",self)

		
		-- Do additional Addon initialization here
	end
end

-----------------------------------------------------------------------------------------------
-- ExportRoster Functions
-----------------------------------------------------------------------------------------------
function ExportRoster:split(s, inSplitPattern)
  local result = { }
  local theStart = 1
  local theSplitStart, theSplitEnd = string.find( s, inSplitPattern, theStart )
  while theSplitStart do
     table.insert( result, string.sub( s, theStart, theSplitStart-1 ) )
     theStart = theSplitEnd + 1
     theSplitStart, theSplitEnd = string.find( s, inSplitPattern, theStart )
  end
  table.insert( result, string.sub( s, theStart ) )
  return result
end

function ExportRoster:GetRosterData(arg) 
	local columnArgs = 'names, classes, roles';
	if (string.len(arg) > 7) then
		columnArgs = string.lower(string.sub(arg, 8));			
	end
	
	local columns = self:split(columnArgs , ',');
	
	groupMembers = {}
	for i=1,GroupLib.GetMemberCount() do
		fields = {};
		for j, v in pairs(columns) do
			v = v:gsub("%s+", "")
			local member = GroupLib.GetGroupMember(i);
			if (string.lower(v) == 'names' or string.lower(v) == 'name') then
				fields[j] = member.strCharacterName;
			elseif (string.lower(v) == 'classes' or string.lower(v) == 'class') then
				fields[j] = Utils.ClassNames[member.eClassId];
			elseif (string.lower(v) == 'roles' or string.lower(v) == 'role') then
				if (member.bTank) then
					fields[j] = 'Tank';
				elseif (member.bHealer) then
					fields[j] = 'Healer';
				else
					fields[j] = 'DPS';
				end
			end
		end
		
		groupMembers[i] = table.concat(fields, ', ');
	end
	
	local data = table.concat(groupMembers, "\r\n")
	if (data == '') then
		data = 'No Group!';
	end
	return data;
end

function ExportRoster:GetItemsText(unit, items) 
	local data = unit:GetName() .. ', ';
	data = data .. Utils.ClassNames[unit:GetClassId()] .. ', ';
	data = data .. unit:GetEffectiveItemLevel() .. ', ';	
	data = data .. unit:GetAssaultPower() .. ', ';
	data = data .. unit:GetSupportPower() .. ', ';
	
	for i, item in ipairs(items) do
		slot = item:GetSlot()
		-- skip tool and key
		if slot ~= 6 and slot ~= 9 and slot < 17 then
			data = data .. item:GetItemId();
			if (slot ~= 16) then
				data = data .. ', ';
			end
		end
	end	
	
	return data;
end

function ExportRoster:GetGearData(arg)
	if (GroupLib.GetMemberCount() == 0) then
		self:ShowData(self:GetItemsText(GameLib.GetPlayerUnit(), GameLib.GetPlayerUnit():GetEquippedItems()));
	else
		self.gearLines = {}
		self.gearLines[1] = self:GetItemsText(GameLib.GetPlayerUnit(), GameLib.GetPlayerUnit():GetEquippedItems());

		self.membersToScan = {}
		self.scanIndex = 0;
		self.scanCount = 0;
		for i=1,GroupLib.GetMemberCount() do
			local member = GroupLib.GetUnitForGroupMember(i);
			
			--make sure the member is valid
			--ignore current player- already have our own gear!
			if (member and member:GetName() ~= GameLib.GetPlayerUnit():GetName()) then
				self.membersToScan[self.scanCount] = member ;
				self.scanCount = self.scanCount + 1;
			end
		end;
		
		if (self.scanIndex == self.scanCount) then
			self:ShowData(table.concat(self.gearLines, "\r\n"));
		else
			self.scanning = true;
		end	
	end
end

function ExportRoster:OnTimer()
	if (self.scanning) then
		self.membersToScan[self.scanIndex]:Inspect();
	end;
end

function ExportRoster:OnInspect(unit, items)
	if (not unit or not self.scanning) then 
		return 
	end
	
	--hide the inspect window if we are doing out thing
	local characterAddOn = Apollo.GetAddon("Character")
	if characterAddOn and characterAddOn.wndInspect then 
		characterAddOn.wndInspect:Show(false) 
	end

	local data = self:GetItemsText(unit, items);

	self.scanIndex = self.scanIndex + 1;
	
	self.gearLines[self.scanIndex + 1] = data;

	if (self.scanIndex >= self.scanCount) then
		self.scanning = false;
		self:ShowData(table.concat(self.gearLines, "\r\n"));
	end	
end	
	
function ExportRoster:ShowData(data)
	if (data ~= '') then
		local clipboardButton = self.wndMain:FindChild("ClipboardButton")
		local text = self.wndMain:FindChild("Text")
		
		text:SetText(data)
		clipboardButton:SetActionData(GameLib.CodeEnumConfirmButtonType.CopyToClipboard, data)
	
		self.wndMain:Show(true)
		self.wndMain:ToFront(true)
	end
end;

-- on SlashCommand "/export"
function ExportRoster:OnExport(cmd, arg)
	local data = '';
	if (string.find(string.lower(arg), 'roster') == 1) then
		self:ShowData(self:GetRosterData(arg));
	elseif (string.find(string.lower(arg), 'gear') == 1) then
		self:GetGearData()
	end
end


-- on SlashCommand "/exportroster"
function ExportRoster:OnExportRoster(cmd, arg)
	self:OnExport(cme, 'roster ' .. arg);
end


-----------------------------------------------------------------------------------------------
-- ExportRosterForm Functions
-----------------------------------------------------------------------------------------------

-- when the Cancel button is clicked
function ExportRoster:OnCancel()
	self.wndMain:Close() -- hide the window
end

-----------------------------------------------------------------------------------------------
-- ExportRoster Instance
-----------------------------------------------------------------------------------------------
local ExportRosterInst = ExportRoster:new()
ExportRosterInst:Init()
